from ._binary_morphology import BinaryClosing, BinaryDilation, BinaryErosion, BinaryOpening
from ._label_morphology import LabelDilation

__all__ = [
    "BinaryClosing",
    "BinaryDilation",
    "BinaryErosion",
    "BinaryOpening",
    "LabelDilation"
]


from ._grey_transform import RGBToGrey
from ._hsv_transform import RGBToHSV
from ._lab_transform import RGBToLAB
from ._od_transform import GreyToOD, RGBToOD

__all__ = [
    "RGBToGrey",
    "RGBToHSV",
    "RGBToLAB",
    "GreyToOD",
    "RGBToOD",
]


from .._process import Process
from dask.array import Array
from dask_image.ndfilters import gaussian_filter

class GaussianFilter(Process):
    def __init__(self, sigma, order=0, mode='reflect', cval=0.0, truncate=4.0):
        super().__init__("Apply Gaussian filter")
        self.sigma = sigma
        self.order = order
        self.mode = mode
        self.cval = cval
        self.truncate = truncate

    def process(self, image: Array) -> Array:
        assert isinstance(image, Array), f"Expected image to be of type Array, got {type(image)}"
        sigma = self.sigma
        order = self.order
        mode = self.mode
        cval = self.cval
        truncate = self.truncate
        return gaussian_filter(image, sigma=sigma, order=order, mode=mode, cval=cval, truncate=truncate)


import dask.array as da
from dask.array import Array
import matplotlib.pyplot as plt
import numpy as np

def show_thumb(image: Array, method: str = "mean", target_size: int = 256) -> None:
    assert isinstance(image, da.Array), f"Invalid image type: {type(image)}"
    assert len(image.shape) == 3 or len(image.shape) == 2, f"Invalid image shape: {image.shape}"
    assert method in ["median", "mean"], f"Invalid method: {method}"
    assert target_size > 0, f"Invalid target size: {target_size}"
    is_threed = len(image.shape) == 3
    if method == "median":
        method = da.median
    else:
        method = da.mean
    coarsen_factor = max(image.shape) // target_size
    if coarsen_factor == 0:
        coarsen_factor = 1
    if is_threed:
        thumb = da.coarsen(method, image, {0: coarsen_factor, 1: coarsen_factor, 2: 1}, trim_excess=True).compute()
        if np.max(thumb) > 1:
            thumb = thumb.astype(np.uint8)
        plt.imshow(thumb)
    else:
        thumb = da.coarsen(method, image, {0: coarsen_factor, 1: coarsen_factor}, trim_excess=True).compute()
        if np.max(thumb) > 1:
            thumb = thumb.astype(np.uint8)
        plt.imshow(thumb, cmap="gray")
    plt.axis("off")
    plt.show()




from ..process._processor import Processor
from abc import ABC, abstractmethod
from dask.array import Array

class Segmenter(ABC):
    def __init__(self, name: str, preprocessor: Processor, postprocessor: Processor) -> None:
        assert isinstance(name, str), "Name must be a string"
        if preprocessor is None:
            preprocessor = self._default_preprocessor()
        if postprocessor is None:
            postprocessor = self._default_postprocessor()
        assert isinstance(preprocessor, Processor), "Preprocessor must be a Processor"
        assert isinstance(postprocessor, Processor), "Postprocessor must be a Processor"
        self.name = name
        self.fitted = False
        self.preprocessor = preprocessor
        self.postprocessor = postprocessor

    @abstractmethod
    def fit(self, image: Array) -> None:
        pass

    @abstractmethod
    def _default_preprocessor(self) -> Processor:
        pass

    @abstractmethod
    def _default_postprocessor(self) -> Processor:
        pass

class PrimarySegmenter(Segmenter):
    def __init__(self, name: str, preprocessor: Processor = None, postprocessor: Processor = None) -> None:
        super().__init__(name, preprocessor, postprocessor)

    @abstractmethod
    def segment(self, image: Array) -> Array:
        pass

    @abstractmethod
    def fit_and_segment(self, image: Array) -> Array:
        pass

class SecondarySegmenter(Segmenter):
    def __init__(self, name: str, preprocessor: Processor = None, postprocessor: Processor = None) -> None:
        super().__init__(name, preprocessor, postprocessor)

    @abstractmethod
    def segment(self, image: Array, mask: Array) -> Array:
        pass

    @abstractmethod
    def fit_and_segment(self, image: Array, mask: Array) -> Array:
        pass

class TertiarySegmenter(Segmenter):
    def __init__(self, name: str, preprocessor: Processor = None, postprocessor: Processor = None) -> None:
        super().__init__(name, preprocessor, postprocessor)

    @abstractmethod
    def segment(self, image: Array, mask_primary: Array, mask_secondary: Array) -> Array:
        pass

    @abstractmethod
    def fit_and_segment(self, image: Array, mask_primary: Array, mask_secondary: Array) -> Array:
        pass